/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.domain.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.yiring.auth.domain.role.Role;
import com.yiring.common.domain.BasicEntity;
import java.io.Serial;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.Comment;

/**
 * 用户
 *
 * @author ifzm
 * 2018/9/3 15:27
 */

@Getter
@Setter
@ToString
@SuperBuilder(toBuilder = true)
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(
    name = "SYS_USER",
    indexes = {
        @Index(columnList = "mobile", unique = true),
        @Index(columnList = "username", unique = true),
        @Index(columnList = "email", unique = true),
        @Index(columnList = "enabled"),
        @Index(columnList = "deleted"),
    }
)
@Comment("系统用户")
public class User extends BasicEntity implements Serializable {

    @Serial
    private static final long serialVersionUID = -5787847701210907511L;

    @Comment("真实姓名")
    String realName;

    @Comment("用户名")
    @Column(unique = true)
    String username;

    @Comment("手机号")
    @Column(unique = true)
    String mobile;

    @Comment("邮箱")
    @Column(unique = true)
    String email;

    @Comment("密码")
    String password;

    @Comment("简介")
    String introduction;

    @Comment("头像")
    String avatar;

    @Comment("是否启用")
    Boolean enabled;

    @Comment("是否删除")
    Boolean deleted;

    @JsonIgnore
    @Builder.Default
    @Comment("角色集合")
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(
        name = "SYS_USER_ROLES",
        joinColumns = { @JoinColumn(name = "user_id") },
        inverseJoinColumns = { @JoinColumn(name = "role_id") }
    )
    Set<Role> roles = new HashSet<>();

    @Comment("最后登录IP地址")
    String lastLoginIp;

    @Comment("最后登录时间")
    LocalDateTime lastLoginTime;
}
